// Copyright (c) 2022 NetEase, Inc. All rights reserved.
// Use of this source code is governed by a MIT license that can be
// found in the LICENSE file.

part of nertc;

class _NERtcAudioMixingMgrImpl extends NERtcAudioMixingManager
    with _SDKLoggerMixin, LoggingApi {
  final _platform = NERtcAudioMixingPlatform.instance;
  final _audioMixingEventCallbacks = <NERtcAudioMixingEventCallback>{};

  _NERtcAudioMixingMgrImpl();

  List<NERtcAudioMixingEventCallback> get audioMixingEventCallbacks =>
      _audioMixingEventCallbacks.toList();

  @override
  String get logTag => 'AudioMixingApi';

  @override
  void setEventCallback(NERtcAudioMixingEventCallback callback) {
    _audioMixingEventCallbacks.add(callback);
  }

  @override
  void removeEventCallback(NERtcAudioMixingEventCallback callback) {
    _audioMixingEventCallbacks.remove(callback);
  }

  @override
  Future<int> getAudioMixingCurrentPosition() async {
    int reply = await wrapper('getAudioMixingCurrentPosition',
        _platform.getAudioMixingCurrentPosition());
    return reply;
  }

  @override
  Future<int> getAudioMixingDuration() async {
    int reply = await wrapper(
        'getAudioMixingDuration', _platform.getAudioMixingDuration());
    return reply;
  }

  @override
  Future<int> getAudioMixingPitch() async {
    int reply =
        await wrapper('getAudioMixingPitch', _platform.getAudioMixingPitch());
    return reply;
  }

  @override
  Future<int> getAudioMixingPlaybackVolume() async {
    int reply = await wrapper('getAudioMixingPlaybackVolume',
        _platform.getAudioMixingPlaybackVolume());
    return reply;
  }

  @override
  Future<int> getAudioMixingSendVolume() async {
    int reply = await wrapper(
        'getAudioMixingSendVolume', _platform.getAudioMixingSendVolume());
    return reply;
  }

  @override
  Future<int> pauseAudioMixing() async {
    int reply = await wrapper('pauseAudioMixing', _platform.pauseAudioMixing());
    return reply;
  }

  @override
  Future<int> resumeAudioMixing() async {
    int reply =
        await wrapper('resumeAudioMixing', _platform.resumeAudioMixing());
    return reply;
  }

  @override
  Future<int> setAudioMixingPitch(int pitch) async {
    apiLogger.i('setAudioMixingPitch#arg{pitch:$pitch}');
    int reply = await wrapper(
        'setAudioMixingPitch', _platform.setAudioMixingPitch(pitch));
    return reply;
  }

  @override
  Future<int> setAudioMixingPlaybackVolume(int volume) async {
    apiLogger.i('setAudioMixingPlaybackVolume#arg{volume:$volume}');
    int reply = await wrapper('setAudioMixingPlaybackVolume',
        _platform.setAudioMixingPlaybackVolume(volume));
    return reply;
  }

  @override
  Future<int> setAudioMixingPosition(int position) async {
    apiLogger.i('setAudioMixingPosition#arg{position:$position}');
    int reply = await wrapper(
        'setAudioMixingPosition', _platform.setAudioMixingPosition(position));
    return reply;
  }

  @override
  Future<int> setAudioMixingSendVolume(int volume) async {
    apiLogger.i('setAudioMixingSendVolume#arg{volume:$volume}');
    int reply = await wrapper(
        'setAudioMixingSendVolume', _platform.setAudioMixingSendVolume(volume));
    return reply;
  }

  @override
  Future<int> startAudioMixing(NERtcAudioMixingOptions options) async {
    apiLogger.i('startAudioMixing#arg{options:$options}');
    int reply =
        await wrapper('startAudioMixing', _platform.startAudioMixing(options));
    return reply;
  }

  @override
  Future<int> stopAudioMixing() async {
    int reply = await wrapper('stopAudioMixing', _platform.stopAudioMixing());
    return reply;
  }
}
